@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004 Brian Tarricone, <brian@tarricone.org>
  *  Copyright (c) 2010 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,16 +29,33 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-
-#include <X11/Xlib.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include <stdarg.h>
 
-#define XFDESKTOP_CHANNEL        "xfce4-desktop"
-#define DEFAULT_BACKDROP         DATADIR "/backgrounds/xfce/xfce-shapes.svg"
-#define DEFAULT_ICON_FONT_SIZE   12
-#define DEFAULT_ICON_SIZE        48
-#define ITHEME_FLAGS             (GTK_ICON_LOOKUP_USE_BUILTIN)
+#define XFDESKTOP_CHANNEL  "xfce4-desktop"
+
+#define MIN_ICON_SIZE               16
+#define MAX_ICON_SIZE              192
+#define DEFAULT_ICON_SIZE           48
+#define MIN_ICON_FONT_SIZE           2
+#define MAX_ICON_FONT_SIZE         144
+#define DEFAULT_ICON_FONT_SIZE      12
+#define MIN_TOOLTIP_ICON_SIZE        0
+#define MAX_TOOLTIP_ICON_SIZE      256
+#define DEFAULT_TOOLTIP_ICON_SIZE   64
+#define MIN_GRAVITY                  0
+#define MAX_GRAVITY                  7
+#define DEFAULT_GRAVITY              0
+
+#define DEFAULT_SINGLE_CLICK        FALSE
+#define DEFAULT_SINGLE_CLICK_ULINE  FALSE
+#define DEFAULT_ICONS_ON_PRIMARY    FALSE
+#define DEFAULT_ICON_FONT_SIZE_SET  FALSE
+#define DEFAULT_ICON_CENTER_TEXT    TRUE
+#define DEFAULT_SHOW_TOOLTIPS       TRUE
+
+#define ITHEME_FLAGS             (GTK_ICON_LOOKUP_USE_BUILTIN | GTK_ICON_LOOKUP_FORCE_SIZE)
 
 #define LIST_TEXT                "# xfce backdrop list"
 #define XFDESKTOP_SELECTION_FMT  "XFDESKTOP_SELECTION_%d"
@@ -54,6 +71,17 @@
 #define SINGLE_WORKSPACE_MODE     "/backdrop/single-workspace-mode"
 #define SINGLE_WORKSPACE_NUMBER   "/backdrop/single-workspace-number"
 
+#define DESKTOP_ICONS_ON_PRIMARY_PROP        "/desktop-icons/primary"
+#define DESKTOP_ICONS_STYLE_PROP             "/desktop-icons/style"
+#define DESKTOP_ICONS_ICON_SIZE_PROP         "/desktop-icons/icon-size"
+#define DESKTOP_ICONS_FONT_SIZE_PROP         "/desktop-icons/font-size"
+#define DESKTOP_ICONS_CUSTOM_FONT_SIZE_PROP  "/desktop-icons/use-custom-font-size"
+#define DESKTOP_ICONS_SHOW_TOOLTIP_PROP      "/desktop-icons/show-tooltips"
+#define DESKTOP_ICONS_TOOLTIP_SIZE_PROP      "/desktop-icons/tooltip-size"
+#define DESKTOP_ICONS_SINGLE_CLICK_PROP      "/desktop-icons/single-click"
+#define DESKTOP_ICONS_SINGLE_CLICK_ULINE_PROP "/desktop-icons/single-click-underline-hover"
+#define DESKTOP_ICONS_GRAVITY_PROP           "/desktop-icons/gravity"
+
 #define DESKTOP_ICONS_SHOW_THUMBNAILS        "/desktop-icons/show-thumbnails"
 #define DESKTOP_ICONS_SHOW_HIDDEN_FILES      "/desktop-icons/show-hidden-files"
 #define DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE "/desktop-icons/file-icons/show-network-removable"
@@ -118,6 +146,13 @@ gint xfdesktop_get_monitor_num(GdkDisplay *display,
 
 gint xfdesktop_get_current_monitor_num(GdkDisplay *display);
 
+gboolean xfdesktop_workspace_get_number_and_total(XfwWorkspaceManager *workspace_manager,
+                                                  XfwWorkspace *workspace,
+                                                  gint *workspace_number,
+                                                  gint *total_workspace_count);
+
+GtkWindow * xfdesktop_find_toplevel(GtkWidget *widget);
+
 void xfdesktop_tree_path_free(gpointer data);
 
 void xfdesktop_widget_unrealize(GtkWidget *widget,
