@@ -2,7 +2,7 @@
  *  xfdesktop
  *
  *  Copyright (c) 2008 Stephan Arts <stephan@xfce.org>
- *  Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2008 Brian Tarricone <brian@tarricone.org>
  *  Copyright (c) 2008 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
  *  Copyright (c) 2011 Jannis Pohlmann <jannis@xfce.org>
  *
@@ -40,26 +40,30 @@
 #include <errno.h>
 #endif
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <cairo-gobject.h>
-#include <gtk/gtk.h>
-#include <gtk/gtkx.h>
-#include <gdk/gdkx.h>
-
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <cairo-gobject.h>
+#include <gtk/gtk.h>
+
+#ifdef ENABLE_X11
+#include <gdk/gdkx.h>
+#include <gtk/gtkx.h>
+#endif  /* ENABLE_X11 */
 
 #include <libxfce4util/libxfce4util.h>
 #include <xfconf/xfconf.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <libwnck/libwnck.h>
+#include <libxfce4windowing/libxfce4windowing.h>
+#ifdef ENABLE_X11
+#include <libxfce4windowing/xfw-x11.h>
+#endif  /* ENABLE_X11 */
 #include <exo/exo.h>
 
 #include "xfdesktop-common.h"
 #include "xfdesktop-thumbnailer.h"
 #include "xfdesktop-settings-ui.h"
-#include "xfdesktop-settings-appearance-frame-ui.h"
 /* for XfceBackdropImageStyle && XfceBackdropColorStyle */
 #include "xfce-backdrop.h"
 
@@ -81,16 +85,6 @@
 #define WINLIST_SHOW_WS_SUBMENUS_PROP        "/windowlist-menu/show-submenus"
 #define WINLIST_SHOW_ADD_REMOVE_WORKSPACES_PROP "/windowlist-menu/show-add-remove-workspaces"
 
-#define DESKTOP_ICONS_ON_PRIMARY_PROP        "/desktop-icons/primary"
-#define DESKTOP_ICONS_STYLE_PROP             "/desktop-icons/style"
-#define DESKTOP_ICONS_ICON_SIZE_PROP         "/desktop-icons/icon-size"
-#define DESKTOP_ICONS_FONT_SIZE_PROP         "/desktop-icons/font-size"
-#define DESKTOP_ICONS_CUSTOM_FONT_SIZE_PROP  "/desktop-icons/use-custom-font-size"
-#define DESKTOP_ICONS_SHOW_TOOLTIP_PROP      "/desktop-icons/show-tooltips"
-#define DESKTOP_ICONS_TOOLTIP_SIZE_PROP      "/desktop-icons/tooltip-size"
-#define DESKTOP_ICONS_SINGLE_CLICK_PROP      "/desktop-icons/single-click"
-#define DESKTOP_ICONS_GRAVITY_PROP           "/desktop-icons/gravity"
-
 typedef struct
 {
     GtkTreeModel *model;
@@ -108,9 +102,9 @@ typedef struct
     gchar *monitor_name;
     gulong image_list_loaded:1;
 
-    WnckWindow *wnck_window;
+    XfwWindow *xfw_window;
     /* We keep track of the current workspace number because
-     * wnck_screen_get_active_workspace sometimes has to return NULL. */
+     * sometimes fetching the active workspace returns NULL. */
     gint active_workspace;
 
     GtkWidget *infobar;
@@ -497,6 +491,8 @@ setup_special_icon_list(GtkBuilder *gxml,
 
     gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(ts));
     g_object_unref(G_OBJECT(ts));
+
+    gtk_tree_view_expand_all(GTK_TREE_VIEW(treeview));
 }
 
 
@@ -710,31 +706,69 @@ xfdesktop_image_list_add_dir(GObject *source_object,
                                              cb_destroy_add_dir_enumeration);
 }
 
+static gint
+workspace_count_total(XfwWorkspaceManager *workspace_manager) {
+    gint n_ws = 0;
+    for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+         l != NULL;
+         l = l->next)
+    {
+        n_ws += xfw_workspace_group_get_workspace_count(XFW_WORKSPACE_GROUP(l->data));
+    }
+    return n_ws;
+}
+
+static XfwWorkspace *
+find_workspace_by_number(XfwWorkspaceManager *workspace_manager,
+                         gint workspace_num)
+{
+    gint total_workspaces = 0;
+    for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+         l != NULL;
+         l = l->next)
+    {
+        XfwWorkspaceGroup *group = XFW_WORKSPACE_GROUP(l->data);
+        gint n_workspaces = xfw_workspace_group_get_workspace_count(group);
+        if (total_workspaces + n_workspaces > workspace_num) {
+            return g_list_nth_data(xfw_workspace_group_list_workspaces(group),
+                                   workspace_num - total_workspaces);
+        }
+    }
+    return NULL;
+}
+
 static void
 xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
-                                              WnckWorkspace *wnck_workspace)
+                                              XfwWorkspace *xfw_workspace)
 {
     gchar buf[1024];
     gchar *workspace_name;
-    WnckScreen *screen;
-    WnckWorkspace *workspace;
+    XfwScreen *screen;
+    XfwWorkspaceManager *workspace_manager;
+    XfwWorkspace *workspace;
 
     /* Don't update the name until we find our window */
-    if(panel->wnck_window == NULL)
+    if (panel->xfw_window == NULL)
         return;
 
     g_return_if_fail(panel->monitor >= 0 && panel->workspace >= 0);
 
-    screen = wnck_window_get_screen(panel->wnck_window);
+    screen = xfw_window_get_screen(panel->xfw_window);
+    workspace_manager = xfw_screen_get_workspace_manager(screen);
 
     /* If it's a pinned window get the active workspace */
-    if(wnck_workspace == NULL) {
-        workspace = wnck_screen_get_workspace(screen, panel->active_workspace);
+    if(xfw_workspace == NULL) {
+        workspace = find_workspace_by_number(workspace_manager, panel->active_workspace);
     } else {
-        workspace = wnck_workspace;
+        workspace = xfw_workspace;
     }
 
-    workspace_name = g_strdup(wnck_workspace_get_name(workspace));
+    if (workspace == NULL) {
+        g_critical("active workspace is not in list");
+        return;
+    }
+
+    workspace_name = g_strdup(xfw_workspace_get_name(workspace));
 
     if(gdk_display_get_n_monitors(gtk_widget_get_display(panel->chk_apply_to_all)) > 1) {
         if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(panel->chk_apply_to_all))) {
@@ -774,7 +808,7 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
         }
     } else {
         if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(panel->chk_apply_to_all)) ||
-           (wnck_screen_get_workspace_count(screen) == 1)) {
+           (workspace_count_total(workspace_manager) == 1)) {
             /* Single monitor and single workspace */
             g_snprintf(buf, sizeof(buf), _("Wallpaper for my desktop"));
 
@@ -925,21 +959,23 @@ cb_image_selection_changed(GtkIconView *icon_view,
 
 static gint
 xfdesktop_settings_get_active_workspace(AppearancePanel *panel,
-                                        WnckWindow *wnck_window)
+                                        XfwWindow *xfw_window)
 {
-    WnckWorkspace *wnck_workspace;
+    XfwWorkspace *workspace;
     gboolean single_workspace;
-    gint workspace_num, single_workspace_num;
-    WnckScreen *wnck_screen = wnck_window_get_screen(wnck_window);
+    gint workspace_num = -1, n_workspaces = 0, single_workspace_num;
+    XfwScreen *xfw_screen = xfw_window_get_screen(xfw_window);
+    XfwWorkspaceManager *workspace_manager = xfw_screen_get_workspace_manager(xfw_screen);
 
-    wnck_workspace = wnck_window_get_workspace(wnck_window);
+    workspace = xfw_window_get_workspace(xfw_window);
 
-    /* If wnck_workspace is NULL that means it's pinned and we just need to
+    /* If workspace is NULL that means it's pinned and we just need to
      * use the active/current workspace */
-    if(wnck_workspace != NULL) {
-        workspace_num = wnck_workspace_get_number(wnck_workspace);
+    if(workspace != NULL) {
+        xfdesktop_workspace_get_number_and_total(workspace_manager, workspace, &workspace_num, &n_workspaces);
     } else {
         workspace_num = panel->active_workspace;
+        n_workspaces = workspace_count_total(workspace_manager);
     }
 
     single_workspace = xfconf_channel_get_bool(panel->channel,
@@ -953,7 +989,7 @@ xfdesktop_settings_get_active_workspace(AppearancePanel *panel,
         single_workspace_num = xfconf_channel_get_int(panel->channel,
                                                       SINGLE_WORKSPACE_NUMBER,
                                                       0);
-        if(single_workspace_num < wnck_screen_get_workspace_count(wnck_screen)) {
+        if(single_workspace_num < n_workspaces) {
             return single_workspace_num;
         } else {
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(panel->chk_apply_to_all),
@@ -1292,7 +1328,7 @@ xfdesktop_settings_update_iconview_folder(AppearancePanel *panel)
     gchar *current_folder, *dirname;
 
     /* If we haven't found our window return now and wait for that */
-    if(panel->wnck_window == NULL)
+    if(panel->xfw_window == NULL)
         return;
 
     TRACE("entering");
@@ -1528,28 +1564,27 @@ suboptions_set_sensitive(GtkToggleButton *btn,
 }
 
 static void
-cb_update_background_tab(WnckWindow *wnck_window,
+cb_update_background_tab(XfwWindow *xfw_window,
                          gpointer user_data)
 {
     AppearancePanel *panel = user_data;
     gint screen_num, monitor_num, workspace_num;
     gchar *monitor_name = NULL;
-    WnckWorkspace *wnck_workspace = NULL;
-    WnckScreen    *wnck_screen = NULL;
+    XfwWorkspace  *workspace = NULL;
     GdkWindow     *window = NULL;
     GdkDisplay    *display = NULL;
     GdkMonitor    *monitor = NULL;
 
     /* If we haven't found our window return now and wait for that */
-    if(panel->wnck_window == NULL)
+    if(panel->xfw_window == NULL)
         return;
 
     /* Get all the new settings for comparison */
-    wnck_screen = wnck_window_get_screen(panel->wnck_window);
-    wnck_workspace = wnck_window_get_workspace(panel->wnck_window);
-
-    workspace_num = xfdesktop_settings_get_active_workspace(panel, wnck_window);
-    screen_num = wnck_screen_get_number(wnck_screen);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    screen_num = gdk_screen_get_number(gtk_widget_get_screen(panel->image_iconview));
+G_GNUC_END_IGNORE_DEPRECATIONS
+    workspace = xfw_window_get_workspace(panel->xfw_window);
+    workspace_num = xfdesktop_settings_get_active_workspace(panel, xfw_window);
     window = gtk_widget_get_window(panel->image_iconview);
     display = gdk_window_get_display(window);
     monitor = gdk_display_get_monitor_at_window(display, window);
@@ -1608,25 +1643,28 @@ cb_update_background_tab(WnckWindow *wnck_window,
     xfdesktop_settings_background_tab_change_bindings(panel,
                                                       FALSE);
 
-    xfdesktop_settings_update_iconview_frame_name(panel, wnck_workspace);
+    xfdesktop_settings_update_iconview_frame_name(panel, workspace);
     xfdesktop_settings_update_iconview_folder(panel);
 }
 
 static void
-cb_workspace_changed(WnckScreen *screen,
-                     WnckWorkspace *workspace,
+cb_workspace_changed(XfwWorkspaceGroup *group,
+                     XfwWorkspace *previously_active_workspace,
                      gpointer user_data)
 {
     AppearancePanel *panel = user_data;
-    WnckWorkspace *active_workspace;
+    XfwWorkspace *active_workspace;
     gint new_num = -1;
 
-    g_return_if_fail(WNCK_IS_SCREEN(screen));
+    g_return_if_fail(group != NULL);
 
     /* Update the current workspace number */
-    active_workspace = wnck_screen_get_active_workspace(screen);
-    if(WNCK_IS_WORKSPACE(active_workspace))
-        new_num = wnck_workspace_get_number(active_workspace);
+    active_workspace = xfw_workspace_group_get_active_workspace(group);
+    if (active_workspace != NULL) {
+        XfwWorkspaceManager *workspace_manager = xfw_workspace_group_get_workspace_manager(group);
+        gint n_ws;
+        xfdesktop_workspace_get_number_and_total(workspace_manager, active_workspace, &new_num, &n_ws);
+    }
 
     /* This will sometimes fail to update */
     if(new_num != -1)
@@ -1635,79 +1673,186 @@ cb_workspace_changed(WnckScreen *screen,
     XF_DEBUG("active_workspace now %d", panel->active_workspace);
 
     /* Call cb_update_background_tab in case this is a pinned window */
-    if(panel->wnck_window != NULL)
-        cb_update_background_tab(panel->wnck_window, panel);
+    if(panel->xfw_window != NULL)
+        cb_update_background_tab(panel->xfw_window, panel);
 }
 
 static void
-cb_workspace_count_changed(WnckScreen *screen,
-                           WnckWorkspace *workspace,
+cb_workspace_group_created(XfwWorkspaceManager *manager,
+                           XfwWorkspaceGroup *group,
+                           AppearancePanel *panel)
+{
+    g_signal_connect(group, "active-workspace-changed",
+                     G_CALLBACK(cb_workspace_changed), panel);
+}
+
+static void
+cb_workspace_count_changed(XfwWorkspaceManager *manager,
+                           XfwWorkspace *workspace,
                            gpointer user_data)
 {
     AppearancePanel *panel = user_data;
 
     /* Update background because the single workspace mode may have
      * changed due to the addition/removal of a workspace */
-    if(panel->wnck_window != NULL)
-        cb_update_background_tab(panel->wnck_window, user_data);
+    if(panel->xfw_window != NULL)
+        cb_update_background_tab(panel->xfw_window, user_data);
+}
+
+static gboolean
+is_our_window(AppearancePanel *panel,
+              XfwWindow *window)
+{
+    gboolean matches = FALSE;
+    GtkWidget *toplevel = gtk_widget_get_toplevel(panel->image_iconview);
+
+#ifdef ENABLE_X11
+    if (xfw_windowing_get() == XFW_WINDOWING_X11) {
+        GdkWindow *toplevel_window = gtk_widget_get_window(toplevel);
+        Window our_xid = gdk_x11_window_get_xid(toplevel_window);
+        Window window_xid = xfw_window_x11_get_xid(window);
+        Window cur_win = our_xid;
+        GdkDisplay *gdpy = gdk_window_get_display(toplevel_window);
+        Display *dpy = gdk_x11_display_get_xdisplay(gdpy);
+        Window root_win = gdk_x11_window_get_xid(gdk_screen_get_root_window(gtk_widget_get_screen(toplevel)));
+
+        // When we're embedded in the settings manager, the XID of 'window' is that of the settings
+        // manager's main window, whereas our XID is the ID of the GtkPlug window.
+        while (cur_win != window_xid && cur_win != root_win) {
+            Window parent_win = 0;
+            Window *children = NULL;
+            unsigned int n_children = 0;
+
+            if (XQueryTree(dpy, cur_win, &root_win, &parent_win, &children, &n_children) != 0) {
+                if (children != NULL) {
+                    XFree(children);
+                    children = NULL;
+                }
+                cur_win = parent_win;
+            } else {
+                cur_win = 0;
+                break;
+            }
+        }
+
+        matches = cur_win == window_xid;
+    } else
+#endif  /* ENABLE_X11 */
+#ifdef ENABLE_WAYLAND
+    if (xfw_windowing_get() == XFW_WINDOWING_WAYLAND) {
+        GtkWindow *toplevel_win = GTK_WINDOW(toplevel);
+        const gchar *window_name = xfw_window_get_name(window);
+        const gchar *toplevel_name = gtk_window_get_title(toplevel_win);
+
+        g_message("No good way to match a GdkWindow and XfwWindow for Wayland; guessing");
+
+        if (window_name != NULL && g_strcmp0(window_name, toplevel_name) == 0
+            && gtk_window_is_maximized(toplevel_win) == xfw_window_is_maximized(window)
+            && ((gdk_window_get_state(gtk_widget_get_window(toplevel)) & GDK_WINDOW_STATE_FULLSCREEN) != 0) == xfw_window_is_fullscreen(window))
+        {
+            matches = TRUE;
+        }
+        // TODO: check window geometry?
+    } else
+#endif  /* ENABLE_WAYLAND */
+    {
+        g_assert_not_reached();
+    }
+
+    return matches;
 }
 
 static void
-cb_window_opened(WnckScreen *screen,
-                 WnckWindow *window,
+cb_window_opened(XfwScreen *screen,
+                 XfwWindow *window,
                  gpointer user_data)
 {
     AppearancePanel *panel = user_data;
-    GtkWidget *toplevel;
-    WnckWorkspace *workspace;
+    XfwWorkspace *workspace;
+    XfwWorkspaceGroup *group = NULL;
 
     /* If we already found our window, exit */
-    if(panel->wnck_window != NULL)
+    if (panel->xfw_window != NULL) {
         return;
-
-    toplevel = gtk_widget_get_toplevel(panel->image_iconview);
+    }
 
     /* If they don't match then it's not our window, exit */
-    if(wnck_window_get_xid(window) != GDK_WINDOW_XID(gtk_widget_get_window(toplevel)))
+    if (!is_our_window(panel, window)) {
         return;
+    }
 
     XF_DEBUG("Found our window");
-    panel->wnck_window = window;
+    panel->xfw_window = window;
 
     /* These callbacks are for updating the image_iconview when the window
      * moves to another monitor or workspace */
-    g_signal_connect(panel->wnck_window, "geometry-changed",
+    g_signal_connect(panel->xfw_window, "geometry-changed",
                      G_CALLBACK(cb_update_background_tab), panel);
-    g_signal_connect(panel->wnck_window, "workspace-changed",
+    g_signal_connect(panel->xfw_window, "workspace-changed",
                      G_CALLBACK(cb_update_background_tab), panel);
 
-    workspace = wnck_window_get_workspace(window);
+    workspace = xfw_window_get_workspace(window);
+    if (workspace == NULL) {
+        XfwWorkspaceManager *workspace_manager = xfw_screen_get_workspace_manager(screen);
+        for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+             l != NULL;
+             l = l->next)
+        {
+            workspace = xfw_workspace_group_get_active_workspace(XFW_WORKSPACE_GROUP(l->data));
+            if (workspace != NULL) {
+                group = XFW_WORKSPACE_GROUP(l->data);
+                break;
+            }
+        }
+    } else {
+        group = xfw_workspace_get_workspace_group(workspace);
+    }
 
-    /* Update the active workspace number */
-    cb_workspace_changed(screen, workspace, panel);
+    if (group != NULL) {
+        /* Update the active workspace number */
+        cb_workspace_changed(group, NULL, panel);
+    } else {
+        g_message("Couldn't find active workspace or group");
+    }
 
     /* Update the background settings */
     cb_update_background_tab(window, panel);
 
-    /* If it's a pinned window get the active workspace */
-    if(workspace == NULL)
-        workspace = wnck_screen_get_workspace(screen, panel->active_workspace);
-
     /* Update the frame name */
     xfdesktop_settings_update_iconview_frame_name(panel, workspace);
 }
 
+#ifdef ENABLE_X11
+static void
+cb_active_window_changed(XfwScreen *screen,
+                         XfwWindow *previously_active_window,
+                         AppearancePanel *panel)
+{
+    XfwWindow *window = xfw_screen_get_active_window(screen);
+
+    if (panel->xfw_window == NULL && window != NULL) {
+        cb_window_opened(screen, window, panel);
+
+        if (panel->xfw_window != NULL) {
+            g_signal_handlers_disconnect_by_func(screen, cb_active_window_changed, panel);
+        }
+    }
+}
+#endif
+
 static void
 cb_monitor_changed(GdkScreen *gscreen,
                    gpointer user_data)
 {
-    AppearancePanel *panel = user_data;
+    if (gdk_display_get_n_monitors(gdk_screen_get_display(gscreen)) > 0) {
+        AppearancePanel *panel = user_data;
 
-    /* Update background because the monitor we're on may have changed */
-    cb_update_background_tab(panel->wnck_window, user_data);
+        /* Update background because the monitor we're on may have changed */
+        cb_update_background_tab(panel->xfw_window, user_data);
 
-    /* Update the frame name because we may change from/to a single monitor */
-    xfdesktop_settings_update_iconview_frame_name(panel, wnck_window_get_workspace(panel->wnck_window));
+        /* Update the frame name because we may change from/to a single monitor */
+        xfdesktop_settings_update_iconview_frame_name(panel, xfw_window_get_workspace(panel->xfw_window));
+    }
 }
 
 static void
@@ -1729,11 +1874,11 @@ cb_xfdesktop_chk_apply_to_all(GtkCheckButton *button,
                                SINGLE_WORKSPACE_NUMBER,
                                panel->workspace);
     } else {
-        cb_update_background_tab(panel->wnck_window, panel);
+        cb_update_background_tab(panel->xfw_window, panel);
     }
 
     /* update the frame name to since we changed to/from single workspace mode */
-    xfdesktop_settings_update_iconview_frame_name(panel, wnck_window_get_workspace(panel->wnck_window));
+    xfdesktop_settings_update_iconview_frame_name(panel, xfw_window_get_workspace(panel->xfw_window));
 }
 
 static void
@@ -1775,6 +1920,22 @@ xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
                      G_CALLBACK(cb_thumbnail_ready), panel);
 }
 
+static void
+cb_icon_style_changed(GtkComboBox *combo,
+                      GtkBuilder *main_gxml)
+{
+    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(main_gxml, "box_icons_appearance_settings")),
+                             gtk_combo_box_get_active(combo) != 0);
+
+#ifdef ENABLE_FILE_ICONS
+    gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(main_gxml, "tab_file_launcher_icons_content")),
+                             gtk_combo_box_get_active(combo) == 2);
+    gtk_widget_set_visible(GTK_WIDGET(gtk_builder_get_object(main_gxml, "infobar_file_launcher_icons_disabled")),
+                           gtk_combo_box_get_active(combo) != 2);
+
+#endif
+}
+
 static void
 cb_xfdesktop_icon_orientation_changed(GtkComboBox *combo,
                                       gpointer user_data)
@@ -1811,14 +1972,16 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
 {
     GtkWidget *appearance_container, *chk_custom_font_size,
               *spin_font_size, *w, *box, *spin_icon_size,
-              *chk_show_thumbnails, *chk_single_click, *appearance_settings,
+              *chk_show_thumbnails, *chk_single_click, *chk_single_click_underline,
+              *appearance_settings,
               *chk_show_tooltips, *spin_tooltip_size, *bnt_exit, *content_area,
               *chk_show_hidden_files;
     GtkBuilder *appearance_gxml;
     GError *error = NULL;
     GtkFileFilter *filter;
     GdkScreen *screen;
-    WnckScreen *wnck_screen;
+    XfwScreen *xfw_screen;
+    XfwWorkspaceManager *workspace_manager;
     XfconfChannel *channel = panel->channel;
     const gchar *path;
     GFile *file;
@@ -1857,6 +2020,11 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     /* single click */
     chk_single_click = GTK_WIDGET(gtk_builder_get_object(main_gxml,
                                                          "chk_single_click"));
+    chk_single_click_underline = GTK_WIDGET(gtk_builder_get_object(main_gxml,
+                                                                   "chk_single_click_underline"));
+    g_object_bind_property(chk_single_click, "active",
+                           chk_single_click_underline, "sensitive",
+                           G_BINDING_SYNC_CREATE);
 
     /* show hidden files */
     chk_show_hidden_files = GTK_WIDGET(gtk_builder_get_object(main_gxml,
@@ -1888,32 +2056,34 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                                   TRUE);
 
     screen = gtk_widget_get_screen(appearance_container);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-    wnck_screen = wnck_screen_get(panel->screen);
-G_GNUC_END_IGNORE_DEPRECATIONS
+    xfw_screen = xfw_screen_get_default();
+    workspace_manager = xfw_screen_get_workspace_manager(xfw_screen);
+    g_signal_connect(workspace_manager, "workspace-group-created",
+                     G_CALLBACK(cb_workspace_group_created), panel);
+    g_signal_connect(workspace_manager, "workspace-created",
+                     G_CALLBACK(cb_workspace_count_changed), panel);
+    g_signal_connect(workspace_manager, "workspace-destroyed",
+                     G_CALLBACK(cb_workspace_count_changed), panel);
 
     /* watch for workspace changes */
-    g_signal_connect(wnck_screen, "workspace-created",
-                     G_CALLBACK(cb_workspace_count_changed), panel);
-    g_signal_connect(wnck_screen, "workspace-destroyed",
-                     G_CALLBACK(cb_workspace_count_changed), panel);
-    g_signal_connect(wnck_screen, "active-workspace-changed",
-                     G_CALLBACK(cb_workspace_changed), panel);
+    for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+         l != NULL;
+         l = l->next)
+    {
+        g_signal_connect(l->data, "active-workspace-changed",
+                         G_CALLBACK(cb_workspace_changed), panel);
+    }
 
     /* watch for window-opened so we can find our window and track it's changes */
-    g_signal_connect(wnck_screen, "window-opened",
+    g_signal_connect(xfw_screen, "window-opened",
                      G_CALLBACK(cb_window_opened), panel);
 
     /* watch for monitor changes */
     g_signal_connect(G_OBJECT(screen), "monitors-changed",
                      G_CALLBACK(cb_monitor_changed), panel);
 
-    appearance_gxml = gtk_builder_new();
-    if(!gtk_builder_add_from_string(appearance_gxml,
-                                    xfdesktop_settings_appearance_frame_ui,
-                                    xfdesktop_settings_appearance_frame_ui_length,
-                                    &error))
-    {
+    appearance_gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-appearance-frame-ui.glade");
+    if (G_UNLIKELY(appearance_gxml == NULL)) {
         g_printerr("Failed to parse appearance settings UI description: %s\n",
                    error->message);
         g_clear_error(&error);
@@ -1921,7 +2091,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     }
 
     appearance_settings = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
-                                                            "alignment_settings"));
+                                                            "background_tab_vbox"));
 
     /* Add the background tab widgets to the main window and don't display the
      * notebook label/tab */
@@ -2096,14 +2266,14 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                            gtk_builder_get_object(main_gxml, "chk_show_winlist_ws_submenus"),
                            "active");
 
-    w = GTK_WIDGET(gtk_builder_get_object(main_gxml, "primary"));
-    xfconf_g_property_bind(channel, DESKTOP_ICONS_ON_PRIMARY_PROP, G_TYPE_BOOLEAN,
-                          G_OBJECT(w), "active");
     w = GTK_WIDGET(gtk_builder_get_object(main_gxml, "combo_icons"));
+    g_signal_connect(w, "changed",
+                     G_CALLBACK(cb_icon_style_changed), main_gxml);
 #ifdef ENABLE_FILE_ICONS
     gtk_combo_box_set_active(GTK_COMBO_BOX(w), 2);
 #else
     gtk_combo_box_set_active(GTK_COMBO_BOX(w), 1);
+    gtk_notebook_remove_page(GTK_NOTEBOOK(gtk_builder_get_object(main_gxml, "notebook_settings")), 3);
 #endif
     xfconf_g_property_bind(channel, DESKTOP_ICONS_STYLE_PROP, G_TYPE_INT,
                            G_OBJECT(w), "active");
@@ -2117,6 +2287,9 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                      G_CALLBACK(cb_xfdesktop_icon_orientation_changed), NULL);
 
     /* bindings */
+    xfconf_g_property_bind(channel, DESKTOP_ICONS_ON_PRIMARY_PROP, G_TYPE_BOOLEAN,
+                           G_OBJECT(gtk_builder_get_object(main_gxml, "primary")),
+                           "active");
     xfconf_g_property_bind(channel, DESKTOP_ICONS_FONT_SIZE_PROP, G_TYPE_DOUBLE,
                            G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin_font_size))),
                            "value");
@@ -2138,9 +2311,12 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     xfconf_g_property_bind(channel, DESKTOP_ICONS_SINGLE_CLICK_PROP,
                            G_TYPE_BOOLEAN, G_OBJECT(chk_single_click),
                            "active");
+    xfconf_g_property_bind(channel, DESKTOP_ICONS_SINGLE_CLICK_ULINE_PROP,
+                           G_TYPE_BOOLEAN, G_OBJECT(chk_single_click_underline),
+                           "active");
 
     setup_special_icon_list(main_gxml, channel);
-    cb_update_background_tab(panel->wnck_window, panel);
+    cb_update_background_tab(panel->xfw_window, panel);
 }
 
 static void
@@ -2231,12 +2407,10 @@ main(int argc, char **argv)
         return 1;
     }
 
+    xfdesktop_settings_ui_register_resource();
 
-    gxml = gtk_builder_new();
-    if(!gtk_builder_add_from_string(gxml, xfdesktop_settings_ui,
-                                    xfdesktop_settings_ui_length,
-                                    &error))
-    {
+    gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-ui.glade");
+    if (G_UNLIKELY(gxml == NULL)) {
         g_printerr("Failed to parse UI description: %s\n", error->message);
         g_clear_error(&error);
         return 1;
@@ -2248,9 +2422,51 @@ main(int argc, char **argv)
     if(opt_enable_debug)
         xfdesktop_debug_set(TRUE);
 
-    if(opt_socket_id == 0) {
-        GtkWidget *dialog;
-        dialog = GTK_WIDGET(gtk_builder_get_object(gxml, "prefs_dialog"));
+#ifdef ENABLE_X11
+    if (opt_socket_id != 0 && xfw_windowing_get() == XFW_WINDOWING_X11) {
+        GtkWidget *plug, *plug_child;
+        XfwScreen *xfw_screen;
+
+        plug = gtk_plug_new(opt_socket_id);
+        gtk_widget_show(plug);
+        g_signal_connect(G_OBJECT(plug), "delete-event",
+                         G_CALLBACK(gtk_main_quit), NULL);
+
+        gdk_notify_startup_complete();
+
+        plug_child = GTK_WIDGET(gtk_builder_get_object(gxml, "notebook_settings"));
+        xfce_widget_reparent(plug_child, plug);
+        gtk_widget_show(plug_child);
+
+        screen = gdk_x11_screen_get_screen_number(gtk_widget_get_screen(plug));
+
+        /* In a GtkPlug setting there isn't an easy way to find our window
+         * in cb_window_opened so we'll just get the screen's active window */
+        xfw_screen = xfw_screen_get_default();
+        panel->xfw_window = xfw_screen_get_active_window(xfw_screen);
+
+        if (panel->xfw_window != NULL) {
+            /* These callbacks are for updating the image_iconview when the window
+             * moves to another monitor or workspace */
+            g_signal_connect(panel->xfw_window, "geometry-changed",
+                             G_CALLBACK(cb_update_background_tab), panel);
+            g_signal_connect(panel->xfw_window, "workspace-changed",
+                             G_CALLBACK(cb_update_background_tab), panel);
+        } else {
+            // XfwScreen doesn't know the active window yet, so wait until it
+            // figures it out
+            g_signal_connect(xfw_screen, "active-window-changed",
+                             G_CALLBACK(cb_active_window_changed), panel);
+        }
+    } else
+#endif  /* ENABLE_X11 */
+    {
+        GtkWidget *dialog = GTK_WIDGET(gtk_builder_get_object(gxml, "prefs_dialog"));
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+        screen = gdk_screen_get_number(gtk_widget_get_screen(dialog));
+G_GNUC_END_IGNORE_DEPRECATIONS
+
         g_signal_connect(dialog, "response",
                          G_CALLBACK(xfdesktop_settings_response),
                          channel);
@@ -2260,43 +2476,12 @@ main(int argc, char **argv)
              xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, -1));
         gtk_window_present(GTK_WINDOW (dialog));
 
-        screen = XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(dialog)));
-
-        /* To prevent the settings dialog to be saved in the session */
-        gdk_x11_set_sm_client_id("FAKE ID");
-
-    } else {
-        GtkWidget *plug, *plug_child;
-        WnckScreen *wnck_screen;
-
-        plug = gtk_plug_new(opt_socket_id);
-        gtk_widget_show(plug);
-        g_signal_connect(G_OBJECT(plug), "delete-event",
-                         G_CALLBACK(gtk_main_quit), NULL);
-
-        gdk_notify_startup_complete();
-
-        plug_child = GTK_WIDGET(gtk_builder_get_object(gxml, "alignment1"));
-        xfce_widget_reparent(plug_child, plug);
-        gtk_widget_show(plug_child);
-
-        screen = XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(plug)));
-
-        /* In a GtkPlug setting there isn't an easy way to find our window
-         * in cb_window_opened so we'll just force wnck to init and get the
-         * active window */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-        wnck_screen = wnck_screen_get(screen);
-G_GNUC_END_IGNORE_DEPRECATIONS
-        wnck_screen_force_update(wnck_screen);
-        panel->wnck_window = wnck_screen_get_active_window(wnck_screen);
-
-        /* These callbacks are for updating the image_iconview when the window
-         * moves to another monitor or workspace */
-        g_signal_connect(panel->wnck_window, "geometry-changed",
-                         G_CALLBACK(cb_update_background_tab), panel);
-        g_signal_connect(panel->wnck_window, "workspace-changed",
-                         G_CALLBACK(cb_update_background_tab), panel);
+#ifdef ENABLE_X11
+        if (xfw_windowing_get() == XFW_WINDOWING_X11) {
+            /* To prevent the settings dialog to be saved in the session */
+            gdk_x11_set_sm_client_id("FAKE ID");
+        }
+#endif  /* ENABLE_X11 */
     }
 
     panel->channel = channel;
