@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include <string.h>
 #endif
 
+#include <math.h>
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -160,6 +161,8 @@ enum
 
 static guint backdrop_signals[LAST_SIGNAL] = { 0, };
 
+static const double backdrop_gamma = 2.2;
+
 /* helper functions */
 
 static GdkPixbuf *
@@ -191,43 +194,111 @@ create_solid(GdkRGBA *color,
     return pix;
 }
 
+static void
+apply_gamma(GdkRGBA *dest,
+            const GdkRGBA *src)
+{
+    dest->red = pow(src->red, backdrop_gamma);
+    dest->green = pow(src->green, backdrop_gamma);
+    dest->blue = pow(src->blue, backdrop_gamma);
+    dest->alpha = src->alpha;
+}
+
+static unsigned int
+round_threshold(double x,
+                double threshold)
+{
+    double ipart;
+    double fpart = modf(x, &ipart);
+    if (fpart >= threshold)
+        return ipart + 1;
+    else
+        return ipart;
+}
+
+static unsigned char
+interpolate(double x1,
+            double x2,
+            int position,
+            int max,
+            double threshold)
+{
+    const double t = (double)position / (max - 1); // we want to interpolate all the way to max-1
+    const double xi = x1 * (1 - t) + x2 * t;
+    return CLAMP((int)round_threshold(pow(xi, 1 / backdrop_gamma) * 255, threshold), 0, 255);
+}
+
+static double gen_threshold(unsigned int *seed)
+{
+    return (double)rand_r(seed) / RAND_MAX;
+}
+
 static GdkPixbuf *
-create_gradient(GdkRGBA *color1, GdkRGBA *color2, gint width, gint height,
-        XfceBackdropColorStyle style)
+create_gradient(GdkRGBA *color1,
+                GdkRGBA *color2,
+                gint width,
+                gint height,
+                XfceBackdropColorStyle style)
 {
     GdkWindow *root;
     GdkPixbuf *pix;
     cairo_surface_t *surface;
-    cairo_pattern_t *pat;
-    cairo_t *cr;
+    gint scale_factor;
+    unsigned char *data;
+    int stride;
+    gint ax1;
+    gint ax1_max;
+    gint ax2;
+    gint ax2_max;
+    GdkRGBA color1_lin;
+    GdkRGBA color2_lin;
+    unsigned int seed;
 
     g_return_val_if_fail(color1 != NULL && color2 != NULL, NULL);
     g_return_val_if_fail(width > 0 && height > 0, NULL);
     g_return_val_if_fail(style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT || style == XFCE_BACKDROP_COLOR_VERT_GRADIENT, NULL);
 
-    root = gdk_screen_get_root_window(gdk_screen_get_default ());
-    surface = gdk_window_create_similar_surface(root, CAIRO_CONTENT_COLOR_ALPHA, width, height);
-    cr = cairo_create(surface);
+    root = gdk_screen_get_root_window(gdk_screen_get_default());
+    scale_factor = gdk_window_get_scale_factor(root);
+    surface = gdk_window_create_similar_image_surface(root, CAIRO_FORMAT_RGB24, width, height, scale_factor);
+    data = cairo_image_surface_get_data(surface);
+    stride = cairo_image_surface_get_stride(surface);
 
-    if(style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) {
-        pat = cairo_pattern_create_linear (0.0, 0.0,  0.0, height);
+    if (style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) {
+        ax1_max = height;
+        ax2_max = width;
     } else {
-        pat = cairo_pattern_create_linear (0.0, 0.0,  width, 0.0);
+        ax1_max = width;
+        ax2_max = height;
     }
 
-    cairo_pattern_add_color_stop_rgba (pat, 1, color2->red, color2->green, color2->blue, color2->alpha);
-    cairo_pattern_add_color_stop_rgba (pat, 0, color1->red, color1->green, color1->blue, color1->alpha);
+    apply_gamma(&color1_lin, color1);
+    apply_gamma(&color2_lin, color2);
+    seed = 0;
 
-    cairo_rectangle(cr, 0, 0, width, height);
-    cairo_set_source(cr, pat);
-    cairo_fill(cr);
-
-    cairo_surface_flush(surface);
+    for (ax1 = 0; ax1 < ax1_max; ax1++) {
+        for (ax2 = 0; ax2 < ax2_max; ax2++) {
+            guint x, y;
+            guint32 *p;
+            unsigned char r, g, b;
+            double threshold = gen_threshold(&seed);
+            r = interpolate(color1_lin.red, color2_lin.red, ax1, ax1_max, threshold);
+            g = interpolate(color1_lin.green, color2_lin.green, ax1, ax1_max, threshold);
+            b = interpolate(color1_lin.blue, color2_lin.blue, ax1, ax1_max, threshold);
+            if (style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) {
+                x = ax2;
+                y = ax1;
+            } else {
+                x = ax1;
+                y = ax2;
+            }
+            p = (guint32 *) (data + y * stride + x * sizeof(guint32));
+            *p = (r << 16) | (g << 8) | (b << 0);
+        }
+    }
+    cairo_surface_mark_dirty(surface);
 
     pix = gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
-
-    cairo_destroy(cr);
-    cairo_pattern_destroy(pat);
     cairo_surface_destroy(surface);
 
     return pix;
