@@ -1,7 +1,7 @@
 /*
  *  xfworkspace - xfce4's desktop manager
  *
- *  Copyright (c) 2004-2007 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2007 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include <gtk/gtk.h>
 #include <xfconf/xfconf.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include "xfce-backdrop.h"
 
@@ -59,12 +60,15 @@ GType xfce_workspace_get_type                     (void) G_GNUC_CONST;
 
 XfceWorkspace *xfce_workspace_new(GdkScreen *gscreen,
                                   XfconfChannel *channel,
+                                  XfwWorkspace *xfw_workspace,
                                   const gchar *property_prefix,
                                   gint number);
 
 gint xfce_workspace_get_workspace_num(XfceWorkspace *workspace);
 void xfce_workspace_set_workspace_num(XfceWorkspace *workspace, gint number);
 
+XfwWorkspace *xfce_workspace_get_xfw_workspace(XfceWorkspace *workspace);
+
 void xfce_workspace_monitors_changed(XfceWorkspace *workspace,
                                      GdkScreen *gscreen);
 
