@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006      Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006      Brian Tarricone, <brian@tarricone.org>
  *  Copyright (c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,16 +36,22 @@
 #include <string.h>
 #endif
 
+#include <gtk/gtk.h>
 
+#ifdef ENABLE_X11
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-
 #include <gdk/gdkx.h>
-#include <gtk/gtk.h>
+#endif  /* ENABLE_X11 */
+
+#ifdef ENABLE_WAYLAND
+#include <gtk-layer-shell.h>
+#endif  /* ENABLE_WAYLAND */
 
 #include <xfconf/xfconf.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include "xfdesktop-common.h"
 #include "xfce-backdrop.h"
@@ -64,9 +70,12 @@ static void xfdesktop_application_finalize(GObject *object);
 static void session_logout(void);
 static void session_die(gpointer user_data);
 
+#ifdef ENABLE_X11
 static void event_forward_to_rootwin(GdkScreen *gscreen, GdkEvent *event);
-
 static gboolean scroll_cb(GtkWidget *w, GdkEventScroll *evt, gpointer user_data);
+static gboolean cb_wait_for_window_manager(gpointer data);
+static void     cb_wait_for_window_manager_destroyed(gpointer data);
+#endif  /* ENABLE_X11 */
 
 static gboolean reload_idle_cb(gpointer data);
 static void cb_xfdesktop_application_reload(GAction  *action,
@@ -94,9 +103,6 @@ static void cb_xfdesktop_application_debug(GAction  *action,
                                            GVariant *parameter,
                                            gpointer  data);
 
-static gboolean cb_wait_for_window_manager(gpointer data);
-static void     cb_wait_for_window_manager_destroyed(gpointer data);
-
 static void xfdesktop_application_startup(GApplication *g_application);
 static void xfdesktop_application_start(XfdesktopApplication *app);
 static void xfdesktop_application_shutdown(GApplication *g_application);
@@ -140,20 +146,22 @@ struct _XfdesktopApplicationClass
 const gchar *fallback_CSS =
 "XfdesktopIconView.view {"
 "	background: transparent;"
-"	color: @theme_selected_fg_color;"
+"}"
+"XfdesktopIconView.view.label {"
+"	background: @theme_bg_color;"
 "	border-radius: 3px;"
-"}"
-"XfdesktopIconView.view:active {"
-"	background: alpha(@theme_selected_bg_color, 0.5);"
-"	text-shadow: 0 1px 1px black;"
-"}"
-"XfdesktopIconView.view .label {"
 "	text-shadow: 1px 1px 2px black;"
 "}"
-"XfdesktopIconView.view .label:active {"
+"XfdesktopIconView.view.label:selected:backdrop {"
+"	background: alpha(@theme_selected_bg_color, 0.5);"
+"	color: @theme_selected_fg_color;"
+"	text-shadow: 0 1px 1px black;"
+"}"
+"XfdesktopIconView.view.label:selected {"
+"	background: @theme_selected_bg_color;"
 "	color: @theme_selected_fg_color;"
 "}"
-"XfdesktopIconView .rubberband {"
+"XfdesktopIconView.rubberband {"
 "	background: alpha(@theme_selected_bg_color, 0.2);"
 "	border: 1px solid @theme_selected_bg_color;"
 "	border-radius: 0;"
@@ -162,6 +170,7 @@ const gchar *fallback_CSS =
 "	color: @theme_selected_bg_color;"
 "}";
 
+
 G_DEFINE_TYPE(XfdesktopApplication, xfdesktop_application, G_TYPE_APPLICATION)
 
 
@@ -288,6 +297,7 @@ session_die(gpointer user_data)
     g_application_quit(G_APPLICATION(app));
 }
 
+#ifdef ENABLE_X11
 static void
 event_forward_to_rootwin(GdkScreen *gscreen, GdkEvent *event)
 {
@@ -353,12 +363,17 @@ event_forward_to_rootwin(GdkScreen *gscreen, GdkEvent *event)
     XSendEvent(dpy, xev2.window, False, ButtonPressMask | ButtonReleaseMask,
             (XEvent *)&xev2);
 }
+#endif  /* ENABLE_X11 */
 
 static gboolean
 scroll_cb(GtkWidget *w, GdkEventScroll *evt, gpointer user_data)
 {
+#ifdef ENABLE_X11
     event_forward_to_rootwin(gtk_widget_get_screen(w), (GdkEvent*)evt);
     return TRUE;
+#else  /* !ENABLE_X11 */
+    return FALSE;
+#endif  /* ENABLE_X11 */
 }
 
 static gboolean
@@ -525,6 +540,7 @@ cb_xfdesktop_application_debug(GAction  *action,
     xfdesktop_debug_set(g_variant_get_boolean(parameter));
 }
 
+#ifdef ENABLE_X11
 /* Cleans up the associated wait for wm resources */
 static void
 wait_for_window_manager_cleanup(WaitForWM *wfwm)
@@ -594,21 +610,24 @@ cb_wait_for_window_manager_destroyed(gpointer data)
 
     wait_for_window_manager_cleanup(wfwm);
 }
+#endif  /* ENABLE_X11 */
 
 static void
 xfdesktop_application_startup(GApplication *g_application)
 {
     XfdesktopApplication *app = XFDESKTOP_APPLICATION(g_application);
-    WaitForWM *wfwm;
-    guint i;
-    gchar **atom_names;
 
     TRACE("entering");
 
     /* hold so it does not exit on us before the main loop gets going */
     g_application_hold(g_application);
 
-    if(!app->opt_disable_wm_check) {
+#ifdef ENABLE_X11
+    if(!app->opt_disable_wm_check && xfw_windowing_get() == XFW_WINDOWING_X11) {
+        WaitForWM *wfwm;
+        guint i;
+        gchar **atom_names;
+
         /* setup data for wm checking */
         wfwm = g_slice_new0(WaitForWM);
         wfwm->dpy = XOpenDisplay(NULL);
@@ -635,7 +654,9 @@ xfdesktop_application_startup(GApplication *g_application)
                                                          cb_wait_for_window_manager,
                                                          wfwm,
                                                          cb_wait_for_window_manager_destroyed);
-    } else {
+    } else
+#endif  /* ENABLE_X11 */
+    {
         /* directly launch */
         xfdesktop_application_start(app);
     }
@@ -739,12 +760,35 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     app->desktop = xfce_desktop_new(gscreen, app->channel, buf);
     g_object_add_weak_pointer(G_OBJECT(app->desktop), (gpointer *)&app->desktop);
 
-    /* hook into the scroll event so we can forward it to the window
-     * manager */
     gtk_widget_add_events(app->desktop, GDK_BUTTON_PRESS_MASK
-                          | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK);
-    g_signal_connect(G_OBJECT(app->desktop), "scroll-event",
-                     G_CALLBACK(scroll_cb), app);
+                          | GDK_BUTTON_RELEASE_MASK);
+    if (xfw_windowing_get() == XFW_WINDOWING_X11) {
+        /* hook into the scroll event so we can forward it to the window
+         * manager */
+        gtk_widget_add_events(app->desktop, GDK_SCROLL_MASK);
+        g_signal_connect(G_OBJECT(app->desktop), "scroll-event",
+                         G_CALLBACK(scroll_cb), app);
+    } else if (xfw_windowing_get() == XFW_WINDOWING_WAYLAND) {
+#ifdef ENABLE_WAYLAND
+        GtkWindow *window = GTK_WINDOW(app->desktop);
+
+        if (!gtk_layer_is_supported()) {
+            g_critical("Your compositor must support the zwlr_layer_shell_v1 protocol");
+            exit(1);
+        }
+
+        gtk_layer_init_for_window(window);
+        gtk_layer_set_layer(window, GTK_LAYER_SHELL_LAYER_BACKGROUND);
+        gtk_layer_set_anchor(window, GTK_LAYER_SHELL_EDGE_TOP, TRUE);
+        gtk_layer_set_anchor(window, GTK_LAYER_SHELL_EDGE_LEFT, TRUE);
+        gtk_layer_set_margin(window, GTK_LAYER_SHELL_EDGE_TOP, 0);
+        gtk_layer_set_margin(window, GTK_LAYER_SHELL_EDGE_LEFT, 0);
+        gtk_layer_set_namespace(window, "desktop");
+#else  /* !ENABLE_WAYLAND */
+        g_critical("xfdesktop was not built with Wayland support");
+        exit(1);
+#endif  /* ENABLE_WAYLAND */
+    }
 
     /* display the desktop and try to put it at the bottom */
     gtk_widget_realize(app->desktop);
@@ -797,8 +841,10 @@ xfdesktop_application_shutdown(GApplication *g_application)
         app->wait_for_wm_timeout_id = 0;
     }
 
-    menu_cleanup();
-    windowlist_cleanup();
+    menu_cleanup(app->channel);
+    windowlist_cleanup(app->channel);
+
+    app->channel = NULL;
 
     if (app->desktop != NULL) {
         gtk_widget_destroy(app->desktop);
