@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,32 +53,22 @@ struct _XfdesktopIconClass
 
     void (*position_changed)(XfdesktopIcon *icon);
 
-    void (*selected)(XfdesktopIcon *icon);
-    /* XfdektopIcon::activated has weird semantics: you should NEVER connect to
-     * this signal normally: always use g_signal_connect_after(), as the default
-     * signal handler may do some special setup for the icon.  this is lame;
-     * you should be able to use normal g_signal_connect(), but signal handlers
-     * with return values are (for some unknown reason) not allowed to be
-     * G_SIGNAL_RUN_FIRST.  go figure. */
-    gboolean (*activated)(XfdesktopIcon *icon);
-
     /*< virtual functions >*/
-    GdkPixbuf *(*peek_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
     const gchar *(*peek_label)(XfdesktopIcon *icon);
+    const gchar *(*peek_tooltip)(XfdesktopIcon *icon);
 
     GdkDragAction (*get_allowed_drag_actions)(XfdesktopIcon *icon);
 
     GdkDragAction (*get_allowed_drop_actions)(XfdesktopIcon *icon, GdkDragAction *suggested_action);
     gboolean (*do_drop_dest)(XfdesktopIcon *icon, GList *src_icons, GdkDragAction action);
 
-    GdkPixbuf *(*peek_tooltip_pixbuf)(XfdesktopIcon *icon, gint width, gint height);
-    const gchar *(*peek_tooltip)(XfdesktopIcon *icon);
-
     gchar *(*get_identifier)(XfdesktopIcon *icon);
 
     void (*set_thumbnail_file)(XfdesktopIcon *icon, GFile *file);
     void (*delete_thumbnail_file)(XfdesktopIcon *icon);
 
+    gboolean (*activate)(XfdesktopIcon *icon,
+                         GtkWindow *window);
     gboolean (*populate_context_menu)(XfdesktopIcon *icon,
                                       GtkWidget *menu);
 };
@@ -87,13 +77,7 @@ GType xfdesktop_icon_get_type(void) G_GNUC_CONST;
 
 /* xfdesktop virtual function accessors */
 
-GdkPixbuf *xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                     gint width,
-                                     gint height);
 const gchar *xfdesktop_icon_peek_label(XfdesktopIcon *icon);
-GdkPixbuf *xfdesktop_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
-                                              gint width,
-                                              gint height);
 const gchar *xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon);
 
 /* returns a unique identifier for the icon, free when done using it */
@@ -114,39 +98,20 @@ gboolean xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
                                      GList *src_icons,
                                      GdkDragAction action);
 
+gboolean xfdesktop_icon_activate(XfdesktopIcon *icon,
+                                 GtkWindow *window);
 gboolean xfdesktop_icon_populate_context_menu(XfdesktopIcon *icon,
                                               GtkWidget *menu);
 
-GtkWidget *xfdesktop_icon_peek_icon_view(XfdesktopIcon *icon);
-
 void xfdesktop_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file);
 void xfdesktop_icon_delete_thumbnail(XfdesktopIcon *icon);
 
-void xfdesktop_icon_invalidate_regular_pixbuf(XfdesktopIcon *icon);
-void xfdesktop_icon_invalidate_tooltip_pixbuf(XfdesktopIcon *icon);
-void xfdesktop_icon_invalidate_pixbuf(XfdesktopIcon *icon);
-
 /*< signal triggers >*/
 
 void xfdesktop_icon_pixbuf_changed(XfdesktopIcon *icon);
 void xfdesktop_icon_label_changed(XfdesktopIcon *icon);
 void xfdesktop_icon_position_changed(XfdesktopIcon *icon);
 
-void xfdesktop_icon_selected(XfdesktopIcon *icon);
-gboolean xfdesktop_icon_activated(XfdesktopIcon *icon);
-void xfdesktop_icon_activated_g_func(gpointer data,
-                                     gpointer user_data);
-
-/*< private-ish; only for use by XfdesktopIconView >*/
-void xfdesktop_icon_set_extents(XfdesktopIcon *icon,
-                                const GdkRectangle *pixbuf_extents,
-                                const GdkRectangle *text_extents,
-                                const GdkRectangle *total_extents);
-gboolean xfdesktop_icon_get_extents(XfdesktopIcon *icon,
-                                    GdkRectangle *pixbuf_extents,
-                                    GdkRectangle *text_extents,
-                                    GdkRectangle *total_extents);
-
 G_END_DECLS
 
 #endif  /* __XFDESKTOP_ICON_H__ */
